import hashlib
import hmac
import os
import uuid


# def encrypt(txt: str, salt: str) -> bytes:
#     # hashlib.sha224(b"Nobody inspects the spammish repetition").hexdigest()
#     key = hashlib.pbkdf2_hmac(
#         'sha256',  # The hash digest algorithm for HMAC
#         txt.encode('utf-8'),  # Convert the password to bytes
#         salt.encode('utf-8'),  # Provide the salt
#         100000  # It is recommended to use at least 100,000 iterations of SHA-256
#     )
#     return key
#
#
# def encrypt_with_new_salt(txt):
#     salt: str = uuid.uuid4().hex
#     return salt, encrypt(txt, salt=salt)
#
#
# def check_key_password(hashed_password: str, txt: str, salt: str):
#     if hmac.compare_digest(encrypt(txt, salt), hashed_password.encode('utf-8')):
#         return True
#     else:
#         return False


def encrypt(txt: str) -> str:
    return hashlib.sha3_256(txt.encode('utf-8')).hexdigest()


def check_hash_password(hash: str, password: str):
    if hmac.compare_digest(encrypt(password), hash):
        return True
    else:
        return False



from email_validator import validate_email, EmailNotValidError


def is_email_valid(email: str,
                   allow_smtputf8: bool = False,
                   check_deliverability: bool = False,
                   allow_empty_local: bool = False
                   ) -> (bool, str):
    try:
        valid = validate_email(email,
                               allow_smtputf8=allow_smtputf8,
                               check_deliverability=check_deliverability,
                               allow_empty_local=allow_empty_local)

        email = valid.email
        return True
    except EmailNotValidError as e:
        return False



import datetime
import pendulum
import math


def date_with_timezone(timezone: str) -> pendulum.DateTime:
    timezone = timezone.replace(' ', '')
    timezone_split = timezone.split('+')
    if len(timezone_split) == 1:
        timezone_split = timezone.split('-')
    if len(timezone_split) != 2:
        return pendulum.now()
    current_date = pendulum.now(tz=timezone_split[0])
    current_date = current_date.add(
        hours=int(timezone_split[1])) if '+' in timezone else current_date.subtract(
        hours=int(timezone_split[1]))
    return current_date


def time_with_timezone(timezone: str):
    date = date_with_timezone(timezone=timezone)
    current_time = date.time()
    return current_time


def date_from_timestamp(timestamp: pendulum.Union[int, float], timezone: str) -> pendulum.DateTime:
    timestamp = math.trunc(timestamp)
    if len(str(timestamp)) == 13:
        timestamp = timestamp/1000
    timezone_split = timezone.split('+')
    if len(timezone_split) == 1:
        timezone_split = timezone.split('-')
    if len(timezone_split) != 2:
        return pendulum.now()
    current_date = pendulum.from_timestamp(timestamp=timestamp, tz=timezone_split[0])
    current_date = current_date.add(
        hours=int(timezone_split[1])) if '+' in timezone else current_date.subtract(
        hours=int(timezone_split[1]))
    return current_date


def pendulum_time_from_time(time: datetime.time) -> pendulum.Time:
    return pendulum.time(hour=time.hour, minute=time.minute, second=time.second, microsecond=time.microsecond)


def time_from_timestamp_with_timezone(timestamp: pendulum.Union[int, float], timezone: str):
    date = date_from_timestamp(timestamp=timestamp, timezone=timezone)
    current_time = date.time()
    return current_time


def today_date_from_time(time: datetime.time, timezone: str = None):
    now = date_with_timezone(timezone=timezone) if timezone else pendulum.now()
    date = pendulum.datetime(year=now.year, month=now.month, day=now.day, hour=int(time.hour),
                             minute=int(time.minute), second=int(time.second), microsecond=int(time.microsecond),
                             tz=now.timezone)
    return date
